@@ -14,15 +14,18 @@ import IPython.display
 from typing import Tuple, Union
 # choose font - can be deactivated
 from matplotlib import rcParams
-from oscovida.data_sources import base_url, hungary_data, jhu_population_url, rki_data, rki_population_url
-from oscovida.plotting_helpers import align_twinx_ticks, cut_dates, has_twin, limit_to_smoothed
+from oscovida.data_sources import base_url, hungary_data, jhu_population_url, rki_data, rki_population_url, rki_population_backup_file
+from oscovida.plotting_helpers import align_twinx_ticks, cut_dates, has_twin, limit_to_smoothed, uncertain_tail
 
 rcParams['font.family'] = 'sans-serif'
-rcParams['font.sans-serif'] = ['Inconsolata']
+rcParams['font.sans-serif'] = ['Tahoma', 'DejaVu Sans', 'Lucida Grande', 'Verdana']
+rcParams['svg.fonttype'] = 'none'
 # need many figures for index.ipynb and germany.ipynb
 rcParams['figure.max_open_warning'] = 50
+from matplotlib.collections import LineCollection
+from matplotlib.colors import LinearSegmentedColormap
+from matplotlib.dates import DateFormatter, date2num, MONDAY, WeekdayLocator
 from matplotlib.ticker import ScalarFormatter, FuncFormatter, FixedLocator
-from matplotlib.dates import DateFormatter, MONDAY, WeekdayLocator
 from bisect import bisect
 
 import matplotlib.pyplot as plt
@@ -136,21 +139,22 @@ def get_country_data_johns_hopkins(country: str,
     assert country in deaths.index, f"{country} not in available countries. These are {sorted(deaths.index)}"
 
     # Some countries report sub areas (i.e. multiple rows per country) such as China, France, United Kingdom
-    # Denmark. In that case, we sum over all regions.
+    # Denmark. In that case, we sum over all regions (by summing over the relevant rows).
+
     tmp = deaths.loc[country]
-    if len(tmp.shape) == 1:
-        d = deaths.loc[country]
+    if len(tmp.shape) == 1:     # most countries (Germany, Italy, ...)
+        d = tmp
     elif len(tmp.shape) == 2:   # China, France, United Kingdom, ...
-        d = deaths.loc[country].sum()
+        d = tmp.drop(columns=['Province/State']).sum()
         d.rename("deaths", inplace=True)
     else:
         raise ValueError("Unknown data set structure for deaths {country}:", tmp)
 
     tmp = cases.loc[country]
     if len(tmp.shape) == 1:
-        c = cases.loc[country]
+        c = tmp
     elif len(tmp.shape) == 2:
-        c = cases.loc[country].sum()
+        c = tmp.drop(columns=['Province/State']).sum()
         c.rename("cases", inplace=True)
     else:
         raise ValueError("Unknown data set structure for cases {country}:", tmp)
@@ -441,16 +445,60 @@ def fetch_csv_data_from_url(source) -> pd.DataFrame:
     return data # type: ignore
 
 
+def _germany_get_population_backup_data_raw() -> pd.DataFrame:
+    """Function is not meant to be used directly.
+    Use germany_get_population() instead (which will call this function if required).
+    """
+
+    # where is the backup file?
+    rki_population_backup_path = os.path.join(os.path.split(__file__)[0],
+                                              rki_population_backup_file)
+
+    population = pd.read_csv(rki_population_backup_path)
+    return population
+
+
+
 @joblib_memory.cache
 def germany_get_population() -> pd.DataFrame:
     """The function's behavior duplicates the one for `germany_get_region()` one."""
     source = rki_population_url
+
     population = fetch_csv_data_from_url(source)
-    population = (
-        population
-        .set_index('county')
-    )
+    try:
+    	population = (
+    	    population
+    	    .set_index('county')
+    	)
+    except KeyError as exception:
+        print(f"Couldn't retrieve population data from RKI ({rki_population_url}). "
+              "Using backup data from August 2020 instead "
+              "(https://github.com/oscovida/oscovida/blob/master/oscovida/backup_data/RKI_Corona_Landkreise.csv)"
+        )
+        population = _germany_get_population_backup_data_raw()
+       	population = (
+    	    population
+    	    .set_index('county')
+    	)
+
     population = population.rename(columns={"EWZ": "population"})
+
+    # Some tidy up of the data:
+
+    # see https://github.com/oscovida/oscovida/issues/210
+    # try to remove this if-clause and see if tests fail:
+    if "LK Saar-Pfalz-Kreis" in population.index:
+        population.loc['LK Saarpfalz-Kreis'] = population.loc['LK Saar-Pfalz-Kreis']
+        population = population.drop('LK Saar-Pfalz-Kreis')
+
+    # 27 July 2021 - test fail because name is "Städteregion Aachen'" in actual data (
+    # i.e. 'StädteRegion' versus 'Städteregion' Aachen)
+    # see https://github.com/oscovida/oscovida/runs/3170956651?check_suite_focus=true#step:6:651
+    if "StädteRegion Aachen" in population.index:
+        population.loc['Städteregion Aachen'] = population.loc['StädteRegion Aachen']
+        population = population.drop('StädteRegion Aachen')
+
+
     return population # type: ignore
 
 
@@ -476,7 +524,14 @@ def population(country: str,
                region: str = None, subregion: str = None) -> Union[int, None]:
     """
     Returns an `int` which corresponds to the population.
-    Only supports regions so far.
+    Only supports JHU countries and Germany Landkreise so far.
+
+    Example:
+
+    $> population(country="Germany", subregion="LK Pinneberg")
+    316103
+    $> population(country="France")
+    65273512
     """
     df = fetch_csv_data_from_url(jhu_population_url)\
         .rename(columns={"Population": "population",
@@ -511,7 +566,11 @@ def population(country: str,
         else:
             if region or subregion:
                 if region in df['region'].values:
-                    return int(df[df['region'] == region].population.sum())
+                    combined_key = f"{region}, {country}"
+                    if combined_key in df['Combined_Key'].values:  # the total population of the region is known
+                        return int(df[df['Combined_Key'] == combined_key].population)
+                    else:   # there's no total population in the dataset, we have to sum up on our own
+                        return int(df[df['region'] == region].population.sum())
                 elif subregion in df['subregion'].values:
                     return int(df.population[subregion])
                 elif region in df['subregion']:    # silently try to use as subregion
@@ -535,7 +594,7 @@ def get_incidence_rates_countries(period=14):
     assert all(cases.columns[:2] == ["Lat", "Long"])
 
     yesterday = datetime.datetime.now() - datetime.timedelta(days=1)
-    fortnight_ago = yesterday - datetime.timedelta(days=period)
+    fortnight_ago = yesterday - datetime.timedelta(days=period+1)
     periods = (fortnight_ago < pd.to_datetime(cases.columns[2:])) & (
         pd.to_datetime(cases.columns[2:]) < yesterday
     )
@@ -736,6 +795,64 @@ def plot_time_step(ax, series, style="-", labels=None, logscale=True):
     return ax
 
 
+def plot_incidence_rate(ax, cases: pd.Series, country: str,
+                        region: str = None, subregion: str = None,
+                        dates: str = None, weeks: int = 0):
+    if dates:
+        cases = cut_dates(cases, dates)
+    if weeks:
+        cases = cases[-7 * (weeks+1):]  # we need to add one week, because we loose one on rolling
+
+    habitants = population(country=country, region=region, subregion=subregion)
+
+    if habitants:
+        incidence = (cases.diff().dropna().rolling(7).sum()/habitants*100000)
+        norm_title = "\n(per 100K people)"
+    else:
+        incidence = (cases.diff().dropna().rolling(7).sum())
+        norm_title = ''
+
+    # convert dates to numbers first
+    inxval = date2num(incidence.index.to_pydatetime())
+    # smoothing
+    if len(incidence.values) > 100:
+        # the number of consecutive segments of the same color, bigger number produces less spaces between segments:
+        chain_len = 2   # 2 means that each segment of the curve is of length 2
+    else:
+        chain_len = 1
+    # a little trick here: we can't split odd amount of points into even number of segments of length 2, therefore
+    # we simply throw away the very first point of the curve making the number of points even
+    starting_point = len(inxval) % chain_len
+    points = np.array([inxval[starting_point:], incidence.values[starting_point:]]).T.reshape(-1, chain_len, 2)
+    segments = np.concatenate([points[:-1], points[1:]], axis=1)
+
+    # colormap: the line is green on the bottom, near zero value, then it's yellow around 30,
+    # then in becomes more red, and above 100 it's totally maroon
+    colors = ["green", "gold", "red", "maroon"]
+    cmap = LinearSegmentedColormap.from_list('RedGreen', colors, len(incidence))
+    norm = plt.Normalize(0, 100)
+    if country == "Germany" and not dates:  # see https://github.com/oscovida/oscovida/issues/228
+        # plot the end of the incidence curve dotted for Germany (the last four points)
+        lc = LineCollection(segments[:-int(4/chain_len)], cmap=cmap, norm=norm, linewidth=LW)
+        lc_end = LineCollection(segments[-int(4/chain_len):], cmap=cmap, norm=norm, linewidth=3, linestyles=[':'], alpha=0.5)
+        lc_end.set_array(incidence.values[-int(4/chain_len)::chain_len])  # set the colors according to y values
+        ax.add_collection(lc_end)
+    else:
+        lc = LineCollection(segments, cmap=cmap, norm=norm, linewidth=LW)
+    lc.set_array(incidence.values[::chain_len])  # set the colors according to y values
+
+    # add collection to axes
+    ax.add_collection(lc)
+    ax.autoscale_view()
+    ax.set_ylabel("7-day incidence rate"+norm_title)
+    ax.yaxis.set_major_formatter(ScalarFormatter())
+
+    x, y = incidence.index[-1], incidence.values[-1]
+    ax.plot([x], [y], marker="+", color=cmap(norm(y)))  # marker at the end of the line
+    ax.annotate(f"{y:.1f}", xy=(x, y * 1.1), color=cmap(norm(y)))   # marker annotation
+    return ax
+
+
 def compute_daily_change(series: pd.Series) -> Tuple[Tuple[pd.Series, str],
                                                      Tuple[pd.Series, str],
                                                      Tuple[pd.Series, str]]:
@@ -785,14 +902,16 @@ def compute_daily_change(series: pd.Series) -> Tuple[Tuple[pd.Series, str],
 
 def plot_daily_change(ax, series: pd.Series, color: str, labels: Tuple[str, str] = None,
                       country: str = None, region: str = None, subregion: str = None,
-                      dates: str = None, weeks: int = 0):
+                      dates: str = None, weeks: int = 0, uncertainty: int = 7):
     """Given a series of data and matplotlib axis ax, plot the
     - difference in the series data from day to day as bars and plot a smooth
     - line to show the overall development
 
     - series is pandas.Series with data as index, and cumulative cases (or
     deaths)
-    - color is color to be used for plotting
+    - `color` is color to be used for plotting
+    - `dates` or `weeks` specify the time range to plot
+    - `uncertainty` is a number of days we plot as a dashed line
 
     See plot_time_step for documentation on other parameters.
     """
@@ -807,6 +926,7 @@ def plot_daily_change(ax, series: pd.Series, color: str, labels: Tuple[str, str]
     if dates:
         change = cut_dates(change, dates)
         smooth2 = cut_dates(smooth2, dates)
+        uncertainty = 1
     if weeks:
         change = change[-7 * weeks:]
         smooth2 = smooth2[-7 * weeks:]
@@ -832,15 +952,17 @@ def plot_daily_change(ax, series: pd.Series, color: str, labels: Tuple[str, str]
         ax1.yaxis.set_major_locator(FixedLocator(ticks))
         ax1.bar(change.index, change.values, color=color,
                 label=ax_label, alpha=bar_alpha, linewidth=LW)
+        # RKI data for deaths in German regions is very untrustworthy,
+        # see https://oscovida.github.io/2020-germany-reporting-delay-meldeverzug.html
+        if country == "Germany" and (region or subregion) and labels[1] == "deaths" and not dates:
+            uncertainty = 6 * 7     # six weeks
 
-        ax1.plot(smooth2.index, smooth2.values, color=color,
+        ax1.plot(smooth2.index[:-uncertainty], smooth2.values[:-uncertainty], color=color,
                  label=ax_label + " " + smooth2_label, linewidth=LW)
-
-        # uncertain graph on the right
-        if not dates:
-            fortnight = series[-14:].diff().dropna()
-            uncert = fortnight.rolling(14, center=True, win_type='gaussian', min_periods=3).mean(std=4)
-            ax1.plot(uncert.index[-7:], uncert.values[-7:], color=color, linestyle='dashed', linewidth=LW, alpha=0.7)
+        # uncertain (dashed) line on the right
+        if uncertainty > 1:
+            uncertain_tail(ax1, series[-2 * uncertainty:].diff().dropna(), days=uncertainty,
+                           color=color, linewidth=LW, alpha=0.7)
 
         ax1.legend()
         ax1.set_ylabel('daily change')
@@ -1108,6 +1230,7 @@ def min_max_in_past_n_days(series, n, at_least = [0.75, 1.25], alert=[0.1, 100],
         n = len(series)
 
     series = series.replace(math.inf, math.nan)
+    series = series.replace(-math.inf, math.nan)
 
     min_ = series[-n:].min() - 0.1    # the -0.1 is to make extra space because the line we draw is thick
     max_ = series[-n:].max() + 0.1
@@ -1325,7 +1448,7 @@ def day0atleast(v0: int, series: pd.Series) -> pd.Series:
     return result
 
 
-def align_sets_at(v0, df):
+def align_sets_at(v0: int, df: pd.DataFrame) -> pd.DataFrame:
     """Accepts data frame, and aligns so that all entries close to v0 are on the same row.
 
     Returns new dataframe with integer index (representing days after v0).
@@ -1443,11 +1566,18 @@ def set_y_axis_limit(data, current_lim):
 
 def make_compare_plot(main_country, compare_with=["Germany", "Australia", "Poland", "Korea, South",
                                                   "Belarus", "Switzerland", "US"],
-                     v0c=10, v0d=3, normalise=True):
+                     v0c=10, v0d=3, normalise=True, align=False):
     rolling = 7
     df_c, df_d = get_compare_data([main_country] + compare_with, rolling=rolling)
-    res_c = align_sets_at(v0c, df_c)
-    res_d = align_sets_at(v0d, df_d)
+
+    if align:
+        res_c = align_sets_at(v0c, df_c)
+        res_d = align_sets_at(v0d, df_d)
+        c_xlabel = f'days since {v0c} cases'
+        d_xlabel = f'days since {v0d} deaths'
+    else:
+        res_c, res_d = df_c, df_d
+        c_xlabel, d_xlabel = None, None
 
     if normalise:
         for country in res_c.keys():
@@ -1464,15 +1594,15 @@ def make_compare_plot(main_country, compare_with=["Germany", "Australia", "Polan
     res_d = res_d.interpolate(method='linear', limit=3)
 
     fig, axes = plt.subplots(2, 1, figsize=(10, 6))
-    ax=axes[0]
+    ax = axes[0]
     norm_str = '\nper 100K people'
-    plot_logdiff_time(ax, res_c, f"days since {v0c} cases",
-                      f"daily new cases{norm_str if normalise else ''}\n(rolling 7-day mean)",
-                      v0=v0c, highlight={main_country:"C1"})
+    plot_logdiff_time(ax, res_c, xaxislabel=c_xlabel,
+                      yaxislabel=f"daily new cases{norm_str if normalise else ''}\n(rolling 7-day mean)",
+                      v0=v0c, highlight={main_country: "C1"}, labels=False)
     ax = axes[1]
-    plot_logdiff_time(ax, res_d, f"days since {v0d} deaths",
-                      f"daily new deaths{norm_str if normalise else ''}\n(rolling 7-day mean)",
-                      v0=v0d, highlight={main_country:"C0"})
+    plot_logdiff_time(ax, res_d, xaxislabel=d_xlabel,
+                      yaxislabel=f"daily new deaths{norm_str if normalise else ''}\n(rolling 7-day mean)",
+                      v0=v0d, highlight={main_country: "C0"}, labels=False)
 
     if not normalise:
         fig.tight_layout(pad=1)
@@ -1595,17 +1725,12 @@ def make_compare_plot_germany(region=None, subregion=None,
         kwargs_d.update({'labels': False})
     elif dates and weeks:
         raise ValueError("`dates` and `weeks` cannot be used together")
-    elif weeks > 0:
+    else:
         res_c = df_c[- weeks * 7:]
         res_d = df_d[- weeks * 7:]
         kwargs_d.update({'yaxislabel': '', 'labels': False})
         kwargs_c.update({'labels': False})
         kwargs_d.update({'labels': False})
-    else:
-        res_c = align_sets_at(v0c, df_c)
-        res_d = align_sets_at(v0d, df_d)
-        kwargs_c.update({'xaxislabel': f"days since {v0c} cases", 'labeloffset': 1})
-        kwargs_d.update({'xaxislabel': f"days since {v0d} deaths", 'labeloffset': 1})
 
     kwargs_c.update({"yaxislabel": "daily new cases\n(rolling 7-day mean)"})
     kwargs_d.update({"yaxislabel": "daily new deaths\n(rolling 7-day mean)"})
@@ -1736,7 +1861,9 @@ def overview(country: str, region: str = None, subregion: str = None,
         raise ValueError("`dates` and `weeks` cannot be used together")
     else:
         c = _c[- weeks * 7:]
-    plot_time_step(ax=axes[0], series=c, style="-C1", labels=(region_label, "cases"))
+    # plot_time_step(ax=axes[0], series=c, style="-C1", labels=(region_label, "cases"))
+    plot_incidence_rate(ax=axes[0], cases=_c,
+                        country=country, region=region, subregion=subregion, dates=dates, weeks=weeks)
     plot_daily_change(ax=axes[1], series=_c, color="C1", labels=(region_label, "cases"),
                       country=country, region=region, subregion=subregion, dates=dates, weeks=weeks)
     # data cleaning
@@ -1750,7 +1877,7 @@ def overview(country: str, region: str = None, subregion: str = None,
             d = cut_dates(_d, dates)
         else:
             d = _d[- weeks * 7:]
-        plot_time_step(ax=axes[0], series=d, style="-C0", labels=(region_label, "deaths"))
+        # plot_time_step(ax=axes[0], series=d, style="-C0", labels=(region_label, "deaths"))
         plot_daily_change(ax=axes[2], series=_d, color="C0", labels=(region_label, "deaths"),
                           country=country, region=region, subregion=subregion, dates=dates, weeks=weeks)
         plot_reproduction_number(axes[4], series=d, color_g="C0", color_R="C4", labels=(region_label, "deaths"))
@@ -1761,6 +1888,7 @@ def overview(country: str, region: str = None, subregion: str = None,
             # combining doubling time plots
             ticks = align_twinx_ticks(ax_dt_c, ax_dt_d)
             ax_dt_d.yaxis.set_major_locator(FixedLocator(ticks))
+            ax_dt_d.grid(False)     # don't draw the second grid on top of the legend
             # create a combined legend
             h_c, l_c = ax_dt_c.get_legend_handles_labels()  # may return [], []
             h_d, l_d = ax_dt_d.get_legend_handles_labels()  # may return [], []
@@ -1778,7 +1906,7 @@ def overview(country: str, region: str = None, subregion: str = None,
         plot_no_data_available(axes[4], mimic_subplot=axes[3], text='R & growth factor (based on deaths)')
 
     # enforce same x-axis on all plots
-    for i in range(1, axes.shape[0]):
+    for i in range(0, axes.shape[0]):
         axes[i].set_xlim(axes[0].get_xlim())
     for i in range(0, axes.shape[0]):
         if not has_twin(axes[i]):
@@ -1787,8 +1915,12 @@ def overview(country: str, region: str = None, subregion: str = None,
         if weeks > 0:
             axes[i].get_xaxis().set_major_locator(WeekdayLocator(byweekday=MONDAY))     # put ticks every Monday
             axes[i].get_xaxis().set_major_formatter(DateFormatter('%d %b'))             # date format: `15 Jun`
+        else:
+            axes[i].xaxis.set_major_formatter(DateFormatter("%b %y"))
+
     week_str = f", last {weeks} weeks" if weeks else ''
-    title = f"Overview {country}{week_str}, last data point from {c.index[-1].date().isoformat()}"
+    region_str = f"{region or subregion}, {country}" if (region or subregion) else country
+    title = f"{region_str}{week_str}, last data point from {c.index[-1].date().isoformat()}"
     axes[0].set_title(title)
 
     # tight_layout gives warnings, for example for Heinsberg
@@ -1802,8 +1934,16 @@ def overview(country: str, region: str = None, subregion: str = None,
 
 def compare_plot(country: str, region: str = None, subregion: str = None,
                  savefig: bool = False, normalise: bool = True,
-                 dates: str = None) -> Tuple[plt.axes, pd.Series, pd.Series]:
+                 dates: str = None, align: bool = False) -> Tuple[plt.axes, pd.Series, pd.Series]:
     """ Create a pair of plots which show comparison of the region with other most suffering countries
+
+    :param country: mandatory
+    :param region: default: None
+    :param subregion: default: None
+    :param savefig: save figure as PDF, default: False
+    :param normalise: whether to normalise data, default: True
+    :param dates: range in the format "2020-03-21:2020-05-21"
+    :param align: whether to align the starting point for different countries, default: False
     """
     c, d = get_country_data(country, region=region, subregion=subregion)
     region_label = get_region_label(country, region=region, subregion=subregion)
@@ -1813,7 +1953,7 @@ def compare_plot(country: str, region: str = None, subregion: str = None,
         d *= 100000 / _population
 
     if not subregion and not region:    # i.e. not a region of Germany
-        axes_compare, res_c, res_d = make_compare_plot(country, normalise=normalise)
+        axes_compare, res_c, res_d = make_compare_plot(country, normalise=normalise, align=align)
 
     elif country == "Germany":   # Germany specific plots
         # On 11 April, Mecklenburg Vorpommern data was missing from data set.
